/* ESP32_pwm_rgbled
 *  
 * Színkeverést játszunk: három potméterrel külön-külön 
 * változtatjuk az RGB LED vörös, zöld és kék színösszetevőit
 * A potméterek csúszkái az A0, A3 és A4 (VP, VN és D32) 
 * analóg bemenetekre csatlakoznak
 * Az RGB LED anódjait a D18, D19, D21 digitális kimenetek vezérlik
 * Áramkorlátozásra egy-egy 220 Ω-os soros ellenállást használunk
 * 
 * A felhasznált program eredetijének forrása:
 * Ravi Teja (Electronics HUB): In-depth ESP32 PWM Tutorial
 * Link: https://www.electronicshub.org/esp32-pwm-tutorial
 * 
 * Arduino 1.8.15 és ESP32 2.0.0 kiadással tesztelve
 */


// LED anódok vezérlését végző kimenetek
const int redLEDPin   = 18;    /* GPIO18 */
const int greenLEDPin = 19;    /* GPIO19 */
const int blueLEDPin  = 21;    /* GPIO21 */

// A kitöltési tényezők tárolói
uint16_t redDutyCycle;
uint16_t greenDutyCycle;
uint16_t blueDutyCycle;

// A PWM csatornák paraméterei
const int redPWMFreq = 5000;   /* 5 KHz */
const int redPWMChannel = 0;
const int redPWMResolution = 12;
const int RED_MAX_DUTY_CYCLE = (int)(pow(2, redPWMResolution) - 1);

const int greenPWMFreq = 5000; /* 5 KHz */
const int greenPWMChannel = 2;
const int greenPWMResolution = 12;
const int GREEN_MAX_DUTY_CYCLE = (int)(pow(2, greenPWMResolution) - 1);

const int bluePWMFreq = 5000;  /*  5 KHz */
const int bluePWMChannel = 4;
const int bluePWMResolution = 12;
const int BLUE_MAX_DUTY_CYCLE = (int)(pow(2, bluePWMResolution) - 1);

const int ADC_RESOLUTION = 4095; /* ADC felbontása: 12-bit */

void setup() {
  /* PWM csatornák konfigurálása (frekvencia, felbontás) */
  ledcSetup(redPWMChannel, redPWMFreq, redPWMResolution);
  ledcSetup(greenPWMChannel, greenPWMFreq, greenPWMResolution);
  ledcSetup(bluePWMChannel, bluePWMFreq, bluePWMResolution);
  /* A LED PWM csatornákat GPIO kiemenetekhez rendeljük */
  ledcAttachPin(redLEDPin, redPWMChannel);
  ledcAttachPin(greenLEDPin, greenPWMChannel);
  ledcAttachPin(blueLEDPin, bluePWMChannel);

}

void loop() {
  /* A három analóg bemenet kiolvasása ADC-vel */
  redDutyCycle = analogRead(A0);
  greenDutyCycle = analogRead(A3);
  blueDutyCycle = analogRead(A4);
  /* A PWM kimenetek beállítása a kívánt kitöltéssel */
  ledcWrite(redPWMChannel, redDutyCycle);
  ledcWrite(greenPWMChannel, greenDutyCycle);
  ledcWrite(bluePWMChannel, blueDutyCycle);
  delay(50);
}
